/* palmlib.h - classes for reading Palm database formats
 */

/*
 * Copyright (c) 2002 Tony Aiuto
 *
 * Permission is granted to anyone to use this software for any purpose on
 * any computer system, and to alter it and redistribute it freely, subject
 * to the following restrictions:
 *
 * 1. Neither the authors nor their employers are responsible for
 *    any consequences of the use of this software.
 *
 * 2. The origin of this software must not be misrepresented, either by
 *    explicit claim or by omission.  Credit to the authors must appear
 *    in documentation and sources.
 *
 * 3. Altered versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 4. This copyright notice may not be removed or altered.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL TONY AIUTO, HIS CURRENT OR PAST EMPLOYERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */


#if !defined(_PALMLIB_H)

#define	_PALMLIB_H	1

#include	<time.h>


class PalmCategory;
class PalmEntry;

class PalmRead {

public:
	PalmRead(FILE *file);
	~PalmRead();

	int loadheader();

	long getSignature() const { return signature; }
	char *getFilename() const { return filename; }
	char *getCustheader() const { return custheader; }

	long getNextFreeCategory() const { return nextFreeCategory; }
	long getCategoryCount() const { return categoryCount; }
	PalmCategory *getCategory(int i) const { return categories[i]; }
	const char *getCategoryName(int i) const;

	long getResourceID() const { return resourceID; }
	long getFieldsPerRow() const { return fieldsPerRow; }
	long getRecordIDPos() const { return recordIDPos; }
	long getRecordStatusPos() const { return recordStatusPos; }
	long getPlacementPos() const { return placementPos; }

	short getFieldCount() const { return fieldCount; }
	short fieldType(int i) { return fieldTypes[i]; }
	long getNumEntries() const { return numEntries; }

	void printheader(FILE *);

	// Get the next record in the file
	PalmEntry *getEntry() const;

	// Should only be accessible to the package and not the
	// outside, but alas, we don't have packages in C++

	long getLongField() const;
	char * getCStringField() const;
	time_t getTimeField() const;
	long getBoolField() const;

	// public members

	// there are none

private:
	long getRawLong() const { return getLong(); }


private:
	FILE	*input;

	unsigned char getByte() const;
	long getShort() const;
	long getLong() const;
	char * getCString() const;

	long	signature;
	char	*filename;
	char	*custheader;

	long	nextFreeCategory;
	long	categoryCount;
	PalmCategory	**categories;

	long resourceID;
	long fieldsPerRow;
	long recordIDPos;
	long recordStatusPos;
	long placementPos;

	short	fieldCount;
	short	*fieldTypes;
	long	numEntries;

	friend class PalmCategory;
	friend class PalmUtil;
	friend class PalmRepeat;
};


class PalmCategory {
public:
	PalmCategory(PalmRead *);
	~PalmCategory();

	void print(FILE *);

public:
	long index;
	long id;
	long dirty;
	char *longname;
	char *shortname;
};



class PhoneNumber {
public:
	PhoneNumber(const PalmRead *);
	~PhoneNumber();

	long labelIndex;
	char *text;
};

class PalmEntry {

public:
	virtual ~PalmEntry();
	virtual void print(const PalmRead *, FILE *) = 0;
	virtual const char *type() = 0;

public:
	long recordID;
	long recStatus;
	long position;

	long isPrivate;
	long categoryIndex;

protected:
	void load(const PalmRead *);

};


class AddressBookEntry : public PalmEntry {

public:
	AddressBookEntry(const PalmRead *);
	~AddressBookEntry();
	const char *type();

	void print(const PalmRead *, FILE *);

	char *lastname;
	char *firstname;
	char *title;
	char *company;

	char *address;
	char *city;
	char *state;
	char *zip;
	char *country;
	char *note;


	PhoneNumber *phoneNumbers[5];
	char	*custom[4];
	long displayIndex;
};

class PalmRepeat {

public:
	PalmRepeat();
	~PalmRepeat();

	void load(const PalmRead *);
	void print(FILE *out);

	long dateException(int i);

public:
	short nDateExceptions;
	long eventFlag;

	short	unknown1;
	char	*classname;
	long brand;
	long interval;
	time_t endDate;
	long dayOfWeek;

	long	day;
	unsigned char	daymask;
	long	weeki;
	long	monthi;

private:
	long	*dateExceptions;
};

class DateBookEntry : public PalmEntry {

public:
	DateBookEntry(const PalmRead *);
	~DateBookEntry();
	const char *type();

	void print(const PalmRead *, FILE *);

public:
	time_t startTime;
	time_t endTime;
	char *name;
	long duration;
	char *note;

	long untimed;

	long alarm;
	long alarmWarn;
	long alarmWarnType;

	PalmRepeat	repeat;
};


class MemoEntry : public PalmEntry {

public:
	MemoEntry(const PalmRead *);
	~MemoEntry();
	const char *type();

	void print(const PalmRead *, FILE *);

public:
	char	*memo;
};


class TodoEntry : public PalmEntry {

public:
	TodoEntry(const PalmRead *);
	~TodoEntry();
	const char *type();

	void print(const PalmRead *, FILE *);

public:
	char	*description;
	char	*note;
	time_t	due;
	long	priority;
	long	completed;
};


/* Utility methods */
class PalmUtil {
public:
	static char *time2str(time_t *theTime);
	static void pbool(FILE *out, long boolval, const char *tag);

	static void dumpCategory(const PalmRead *reader, FILE *out);
	static void dumpField(const PalmRead *reader, FILE *out);
	static void dumpEntry(const PalmRead *reader, FILE *out);

	static char *phoneTypes[];
	static int nPhoneTypes();
};



/* File signatures */
#define	PALM_DBA_SIG	0x44420100	
#define	PALM_ABA_SIG	0x41420100
#define PALM_ABX_SIG	0x464c457f
#define	PALM_MEMO_SIG	0x4d500100
#define	PALM_TODO_SIG	0x54440100

/* data field types */
#define	PALM_NONE	0
#define	PALM_LONG	1
#define	PALM_FLOAT	2
#define	PALM_TIME	3
#define	PALM_ALPHA	4
#define	PALM_CSTRING	5
#define	PALM_BOOL	6
#define	PALM_BITFLAG	7
#define	PALM_REPEAT	8

#endif	/* PALMLIB_H */

